<?php

namespace EventPortal\Http\Controllers\Admin;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Domain\Team\TeamListing;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\Render;
use EventPortal\Http\Controllers\Controller;
use EventPortal\Http\Requests\TeamRequest;
use Illuminate\Http\RedirectResponse;
use Inertia\Response;

class TeamController extends Controller
{
    public function index(): Response
    {
        $table = Table::make('team.index')
            ->query(Team::query())
            ->request(request())
            ->get();

        return Render::component('Team/Index')
            ->key('team.index')
            ->withTable($table)
            ->get();
    }

    public function create(): Response
    {
        $admins = Admin::all()->mapInto(TeamListing::class);

        $breadcrumbs = Breadcrumb::make()
            ->add('Teams', route('admin.team.index'))
            ->add('Create a team')
            ->get();

        return Render::component('Team/Create')
            ->key('team.index')
            ->withBreadcrumbs($breadcrumbs)
            ->attributes(['allUsers' => $admins])
            ->get();
    }

    public function store(TeamRequest $request): RedirectResponse
    {
        $team = Team::create($request->only([
            'name',
            'description',
        ]));

        $team->admins()->attach($request->input('members'));

        return redirect()->route('admin.team.index');
    }

    public function edit(Team $team): Response
    {
        $teamUsers = $team->admins()->get();
        $allUsers = Admin::query()
            ->whereNotIn('user_id', $teamUsers->pluck('id'))
            ->get();

        $breadcrumbs = Breadcrumb::make()
            ->add('Teams', route('admin.team.index'))
            ->add($team->name, route('admin.team.edit', $team))
            ->get();

        return Render::component('Team/Edit')
            ->key('team.edit')
            ->attributes([
                'teamName' => $team->name,
                'teamId' => $team->id,
                'allUsers' => $allUsers->mapInto(TeamListing::class)->values()->toArray(),
                'teamUsers' => $teamUsers->mapInto(TeamListing::class)->values()->toArray(),
            ])
            ->withBreadcrumbs($breadcrumbs)
            ->get();
    }

    public function update(TeamRequest $request, Team $team): RedirectResponse
    {
        $team->update($request->only([
            'slug',
            'name',
            'description',
        ]));

        $team->admins()->sync($request->input('members'));

        return redirect()->route('admin.team.index');
    }
}
